unit pdfcontrolpanel;

{$mode ObjFPC}

interface

uses
  Classes, SysUtils, web,  pdfpanel;

Const
  ScaleStep = 0.2;


Type

  { TPDFControlPanel }

  TPDFControlPanel = Class(TComponent)
  private
    FEditPageD: string;
    FEditPageID: string;
    FFirstButtonID: string;
    FLastButtonID: string;
    FNextButtonID: string;
    FPDFPanel: TPDFPanel;
    edtPageNo : TJSHTMLInputElement;
    btnZoomIn,
    btnZoomOut,
    btnPrevious,
    btnFirst,
    btnNext,
    btnLast : TJSHTMLButtonElement;
    FPreviousButtonID: string;
    FZoomInButtonID: string;
    FZoomLabelID: string;
    FZoomOutButtonID: string;
    lblZoom : TJSHTMLElement;
    FPageNum : Integer;
    procedure DoPageLoaded(Sender: TObject);
    procedure SetPDFPanel(AValue: TPDFPanel);
  protected
    // Navigation
    function DoPageKeyUp(aEvent: TJSKeyBoardEvent): boolean;
    procedure onFirstPage(aEvent: TJSEvent);
    procedure onLastPage(aEvent: TJSEvent);
    procedure onNextPage(aEvent: TJSEvent);
    procedure onPrevPage(aEvent: TJSEvent);
    // Zoom
    procedure onZoomIn(aEvent: TJSEvent);
    procedure onZoomOut(aEvent: TJSEvent);
    procedure DisplayZoom;
  Public
    constructor Create(aOwner : TComponent); override;
    Procedure BindElements;
    property PDFPanel : TPDFPanel Read FPDFPanel Write SetPDFPanel;
    Property PageNum : Integer Read FPageNum;
    Property PreviousButtonID : string Read FPreviousButtonID Write FPreviousButtonID;
    Property NextButtonID : string Read FNextButtonID Write FNextButtonID;
    Property FirstButtonID : string Read FFirstButtonID Write FFirstButtonID;
    Property LastButtonID : string Read FLastButtonID Write FLastButtonID;
    Property EditPageID : string Read FEditPageID Write FEditPageID;
    Property ZoomInButtonID : string Read FZoomInButtonID Write FZoomInButtonID;
    Property ZoomOutButtonID : string Read FZoomOutButtonID Write FZoomOutButtonID;
    Property ZoomLabelID : string Read FZoomLabelID Write FZoomLabelID;
  end;

implementation

{ TPDFControlPanel }

procedure TPDFControlPanel.onPrevPage(aEvent : TJSEvent);

begin
  if (PageNum <= 1) then exit;
  Dec(FpageNum);
  FPDFPanel.queueRenderPage(FpageNum);
end;

procedure TPDFControlPanel.SetPDFPanel(AValue: TPDFPanel);
begin
  if FPDFPanel=AValue then Exit;
  FPDFPanel:=AValue;
  FPDFPanel.OnPageLoaded:=@DoPageLoaded;
end;

procedure TPDFControlPanel.DoPageLoaded(Sender: TObject);
begin
  FPageNum:=FPDFPanel.DisplayedPage;
  edtPageNo.Value:=IntToStr(FPageNum);
end;

function TPDFControlPanel.DoPageKeyUp(aEvent: TJSKeyBoardEvent): boolean;
Var
  aPage : Integer;

begin
  Result:=False;
  if not assigned(PDFPanel) then
    exit;
  if (aEvent.Key<>TJSKeyNames.Enter) then
    exit;
  aPage:=StrToIntDef(edtPageNo.Value,0);
  if (aPage>0) and (aPage<=PdfPanel.PageCount) then
    PdfPanel.QueueRenderPage(aPage);
end;

procedure TPDFControlPanel.onFirstPage(aEvent : TJSEvent);

begin
  if not assigned(PdfPanel) then
    exit;
  FPageNum:=1;
  PdfPanel.queueRenderPage(FpageNum);
end;

procedure TPDFControlPanel.onLastPage(aEvent : TJSEvent);

begin
  if not assigned(PdfPanel) then
    exit;
  FPageNum:=PdfPanel.PageCount;
  PdfPanel.queueRenderPage(FpageNum);
end;

procedure TPDFControlPanel.onNextPage(aEvent : TJSEvent);

begin
  if not assigned(PdfPanel) then
    exit;
  if (FpageNum >= PdfPanel.PageCount) then
    exit;
  Inc(FpageNum);
  PdfPanel.queueRenderPage(FpageNum);
end;

procedure TPDFControlPanel.onZoomIn(aEvent : TJSEvent);

begin
  if not assigned(PdfPanel) then
    exit;
  PDFPanel.Scale:=PDFPanel.Scale+ScaleStep;
  DisplayZoom;

end;

procedure TPDFControlPanel.onZoomOut(aEvent : TJSEvent);

begin
  if not assigned(PdfPanel) then
    exit;
  if PDFPanel.Scale<=ScaleStep*2 then
    exit;
  PDFPanel.Scale:=PDFPanel.Scale-ScaleStep;
  DisplayZoom;
  PdfPanel.queueRenderPage(FpageNum);
end;

constructor TPDFControlPanel.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FEditPageID:='edtPageNo';
  FFirstButtonID:='btnFirst';
  FLastButtonID:='btnLast';
  FNextButtonID:='btnNext';
  FPreviousButtonID:='btnPrevious';
  FZoomInButtonID:='btnZoomIn';
  FZoomOutButtonID:='btnZoomOut';
  FZoomLabelID:='lblZoom';
end;


procedure TPDFControlPanel.DisplayZoom;
begin
  lblZoom.innerText:=IntToStr(Round(PDFPanel.Scale*100))+'%';
end;

procedure TPDFControlPanel.BindElements;

  function GetHTMLElement(aID : String) : TJSHTMLElement;
  begin
    Result:=TJSHTMLElement(document.getElementById(aID));
    if Not Assigned(Result) then
      Writeln('Element ',aId,' Not found');
  end;

begin
  // Navigation
  btnPrevious:=TJSHTMLButtonElement(GetHTMLElement(PreviousButtonID));
  btnPrevious.addEventListener('click', @onPrevPage);
  btnFirst:=TJSHTMLButtonElement(GetHTMLElement(FirstButtonID));
  btnFirst.addEventListener('click', @onFirstPage);
  btnNext:=TJSHTMLButtonElement(GetHTMLElement(NextButtonID));
  btnNext.addEventListener('click', @onNextPage);
  btnLast:=TJSHTMLButtonElement(GetHTMLElement(LastButtonID));
  btnLast.addEventListener('click', @onLastPage);
  edtPageNo:=TJSHTMLInputElement(GetHTMLElement(EditPageID));
  edtPageNo.onkeyup:=@DoPageKeyUp;
  // Zoom
  btnZoomIn:=TJSHTMLButtonElement(GetHTMLElement(ZoomInButtonID));
  btnZoomIn.addEventListener('click', @onZoomIn);
  btnZoomOut:=TJSHTMLButtonElement(GetHTMLElement(ZoomOutButtonID));
  btnZoomOut.addEventListener('click', @onZoomOut);
  lblZoom:=GetHTMLElement(ZoomLabelID);
  DisplayZoom;
end;

end.

