program pdfindex;

{$mode objfpc}
{$modeswitch advancedrecords}

uses
  browserapp, JS, Classes, SysUtils, Web, pdfjs, searchpdf, fieldmap,
  pdfpanel, pdfcontrolpanel, pdfsearchpanel ;

var
  pdfjsLib : TPDFJSStatic; external name 'pdfjsLib';

Type
  { TPDFIndexApp }

  TPDFIndexApp = class(TBrowserApplication)
    pnlSidebar,
    lblFileLocation : TJSHTMLElement;
    btnLoad,
    btnUpload,
    btnCloseSidebar : TJSHTMLButtonElement;
    edtPDFFile: TJSHTMLInputElement;
    FPDFPanel : TPDFPanel;
    FPDFControls : TPDFControlPanel;
    FPDFSearch: TPDFSearchControl;
    procedure doRun; override;
    procedure BindElements;
  private
    // Load
    function DoLoadFile(Event: TEventListenerEvent): boolean;
    procedure onLoad(aEvent: TJSEvent);
    procedure ShowPDF(aSource: TPDFSource);
    Procedure DisplayFileLocation(const aLocation : String);
    // Search
    procedure onClosePane(aEvent: TJSEvent);
  end;

{ TRestDataset }



procedure TPDFIndexApp.DisplayFileLocation(const aLocation: String);
begin
  if aLocation<>'' then
    begin
    document.title:='View PDF file: '+aLocation;
    lblFileLocation.InnerText:=': '+aLocation;
    end
  else
    begin
    document.title:='View PDF file';
    lblFileLocation.InnerText:='';
    end;
end;


function TPDFIndexApp.DoLoadFile(Event: TEventListenerEvent): boolean;

Var
  aFileName : string;

  Function LoadFromFile(aData : JSValue) : JSValue;

  var
    aSource : TPDFSource;

  begin
    Result:=False;
    aSource:=TPDFSource.New;
    aSource.data:=aData;
    ShowPDF(aSource);
    DisplayFileLocation(aFileName);
  end;

begin
  Result:=False;
  if (edtPDFFile.files.length=1) then
    begin
    aFileName:=ExtractFileName(edtPDFFile.files[0].Name);
    edtPDFFile.files[0].arraybuffer._then(@loadFromFile);
    end;
end;


procedure TPDFIndexApp.ShowPDF(aSource: TPDFSource);

begin
  FPDFPanel.ShowPDF(aSource,1);
end;

procedure TPDFIndexApp.doRun;

const
  TheURL = 'https://mozilla.github.io/pdf.js/build/pdf.worker.js';

begin
  pdfjsLib.GlobalWorkerOptions.workerSrc:=TheURL;
  FPDFPanel:=TPDFPanel.Create(Self);
  FPDFControls:=TPDFControlPanel.Create(Self);
  FPDFControls.PDFPanel:=FPDFPanel;
  FPDFSearch:=TPDFSearchControl.Create(Self);
  FPDFSearch.PDFPanel:=FPDFPanel;
  BindElements;
  Terminate;
end;


procedure TPDFIndexApp.BindElements;
begin
  btnLoad:=TJSHTMLButtonElement(GetHTMLElement('btnLoad'));
  btnLoad.addEventListener('click', @onLoad);
  lblFileLocation:=GetHTMLElement('lblFileLocation');
  edtPDFFile:=TJSHTMLInputElement(GetHTMLElement('edtPDFFile'));
  edtPDFFile.onchange:=@DoLoadFile;
  btnCloseSidebar:=TJSHTMLButtonElement(GetHTMLElement('btnClosePane'));
  btnCloseSidebar.addEventListener('click', @onClosePane);
  pnlSidebar:=TJSHTMLInputElement(GetHTMLElement('pnlSidebar'));
  FPDFPanel.CanvasID:='PDFCanvas'; // Will call bindelements
  FPDFControls.BindElements;
  FPDFSearch.BindElements
end;

procedure TPDFIndexApp.onLoad(aEvent: TJSEvent);

var
  aSource : TPDFSource;
  aURL : String;

begin
  aUrl:=window.prompt('Please enter the URL of a PDF to view');
  if IsNull(aURL) then
    exit;
  aSource:=TPDFSource.New;
  aSource.URL:=aURL;
  ShowPdf(aSource);
  DisplayFileLocation(aURL);
end;

procedure TPDFIndexApp.onClosePane(aEvent : TJSEvent);

begin
  if pnlSidebar.classList.contains('sidebar_closed') then
    FPDFSearch.ShowResultPanel
  else
    FPDFSearch.HideResultPanel;
end;


var
  Application : TPDFIndexApp;

begin
  Application:=TPDFIndexApp.Create(nil);
  Application.Initialize;
  Application.Run;
end.
