unit pdfpanel;

{$mode ObjFPC}

interface

uses
  classes, web, pdfjs;

var
  pdfjsLib : TPDFJSStatic; external name 'pdfjsLib';

Type

  { TPDFPanel }

  TPDFPanel = Class(TComponent)
  private
    FCanvasID: String;
    FDisplayedPage: Integer;
    FOnLoaded: TNotifyEvent;
    FOnPageLoaded: TNotifyEvent;
    FPageCount: Integer;
    FpdfDoc : TPDFDocumentProxy;
    FPageRendering : Boolean;
    FPageNumPending : Integer;
    Fscale : Double ; // 0.8,
    FCanvas : TJSHTMLCanvasElement;
    FCtx : TJSCanvasRenderingContext2D; // canvas.getContext('2d');
    function GetPageCount: Integer;
    procedure SetCanvasID(AValue: String);
    procedure SetScale(AValue: Double);
  Protected
    Procedure renderPage(aNum : Integer);
    Procedure BindElement;
  public
    constructor Create(aOwner : TComponent); override;
    Procedure QueueRenderPage(aNum : Integer);
    Procedure ShowPDF(aSource : TPDFSource; AtPage : Integer = 1);
    Property CanvasID : String Read FCanvasID Write SetCanvasID;
    Property PageRendering : Boolean Read FPageRendering;
    Property Scale : Double Read FScale Write SetScale;
    Property DisplayedPage : Integer Read FDisplayedPage ;
    Property OnPageLoaded : TNotifyEvent Read FOnPageLoaded Write FOnPageLoaded;
    Property PageCount : Integer Read GetPageCount;
    Property OnLoaded : TNotifyEvent Read FOnLoaded Write FOnLoaded;
    Property PDFDoc : TPDFDocumentProxy Read FPDFDoc;
  end;

implementation

{ TPDFPanel }

procedure TPDFPanel.SetCanvasID(AValue: String);
begin
  if FCanvasID=AValue then Exit;
  FCanvasID:=AValue;
  BindElement;
end;

function TPDFPanel.GetPageCount: Integer;
begin
  if Assigned(pdfDoc) then
    Result:=pdfDoc.numPages
  else
    Result:=0;
end;

procedure TPDFPanel.SetScale(AValue: Double);
begin
  if FScale=AValue then Exit;
  FScale:=AValue;
  if (FDisplayedPage>0) and Assigned(pdfDoc) then
    QueueRenderPage(FDisplayedPage);
end;

procedure TPDFPanel.QueueRenderPage(aNum: Integer);
begin
  if FPageRendering then
    FpageNumPending:=aNum
  else
    renderPage(aNum)
end;

procedure TPDFPanel.BindElement;
begin
  FCanvas:=TJSHTMLCanvasElement(document.getElementById(FCanvasID));
  FCtx:=TJSCanvasRenderingContext2D(FCanvas.getContext('2d'));
end;

constructor TPDFPanel.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FScale:=1;
  FPageNumPending:=-1;
end;

procedure TPDFPanel.renderPage(aNum: Integer);

  function renderOK(aValue : JSValue) : JSValue;

  Var
    N : Integer;

  begin
    FPageRendering:=false;
    if (FPageNumPending <> -1) then
      begin
      N:=FPageNumPending;
      FpageNumPending:=-1;
      renderPage(N);
      end;
    Result:=True;
  end;

  function havePage (aValue : JSValue) : JSValue;

  var
    page : TPDFPageProxy absolute aValue;
    viewport : TPDFPageViewport;
    renderContext: TPDFRenderParams;
    renderTask : TPDFRenderTask;
    viewportParams : TViewportParameters;

  begin
    viewportParams:=TViewportParameters.new;
    viewportParams.scale:=FScale;
    viewport:=page.getViewport(viewportParams);
    Fcanvas.height := viewport.height;
    Fcanvas.width := viewport.width;
    renderContext:=TPDFRenderParams.New;
    renderContext.canvasContext:=Fctx;
    renderContext.viewport:=viewport;
    renderTask:=page.render(renderContext);
    renderTask.promise.&then(@renderOK);
    FDisplayedPage:=aNum;
    if Assigned(FOnPageLoaded) then
      FOnPageLoaded(Self);
    Result:=True;
  end;

begin
  Writeln('Rendering page : ',aNum);
  FpageRendering:=True;
  pdfDoc.getPage(aNum).&then(@HavePage);
//  edtPageNo.Value:=IntToStr(anum);
end;


procedure TPDFPanel.ShowPDF(aSource: TPDFSource; AtPage: Integer);

  function pdfLoaded(aValue : JSValue) : JSValue;

  begin
    FPDFDoc:=TPDFDocumentProxy(aValue);
    QueueRenderPage(atPage);
    Result:=True;
    If Assigned(OnLoaded) and (aSource.url<>'') then
      FonLoaded(Self);
  end;

  Procedure pdfLoadError(aReason : String);

  begin
    Writeln('PDF loading failed: '+aReason);
  end;

var
  loadingTask : TPDFLoadingTask;

begin
  Writeln('Showing ',aSource.url,' at ',atPage);
  FPDFDoc:=Nil;
  loadingTask:=pdfjsLib.getDocument(aSource);
  loadingTask.promise.&then(@pdfLoaded,@pdfLoadError);
end;

end.

