unit pdfsearchpanel;

{$mode objfpc}
{$modeswitch advancedrecords}

interface

uses
  Classes, SysUtils, js, web, pdfpanel, pdfjs, restconnection, db, ExtJSDataset, searchpdf, fieldmap;

Const
  ServerURL = 'http://localhost:3010/';



Type
  TServerResultsMap = CLass(TFieldMap)
    FID : TField;
    FRank : TField;
    FPage  : TField;
    FIssue  : TField;
    FAuthor  : TField;
    FTitle  : TField;
    FURL  : TField;
    FContext  : TField;
    FDate  : TField;
    Procedure InitFields; override;
  end;

  { TServerMatch }

  TServerMatch = record
    ID : Integer;
    Rank : Integer;
    Page : Integer;
    Issue : String;
    Author : String;
    Title : string;
    URL : String;
    Context : String;
    Date : TDateTime;
    Procedure FromMap(aMap : TServerResultsMap);
  end;

  { TRestDataset }

  TRestDataset = Class(TExtJSJSONObjectDataSet)
  private
    FConnection: TRestConnection;
  Protected
    Function DoGetDataProxy: TDataProxy; override;
  Public
    Property Connection: TRestConnection Read FConnection Write FConnection;
  end;

  { TPDFSearchControl }

  TPDFSearchControl = Class(TComponent)
  private
    FDivAutoCompleteID: String;
    FLocalCheckboxID: String;
    FpdfPanel: TPDFPanel;
    FResultPanelID: String;
    FSearch: TPDFSearch;
    FConn : TRestConnection;
    FResult : TRestDataset;
    FSearchButtonID: String;
    FSearchEditID: String;
    fShowResultsPanelButtonID: String;
    FSidebarPanelID: String;
    FWords : TRestDataset;
    FSearchTerm : String;
    FSearchTimerID : Integer;
    btnSearch : TJSHTMLButtonElement;
    cbLocal,
    edtSearch : TJSHTMLinputElement;
    btnClosePaneImage,
    pnlSidebar,
    pnlResults,
    mnuAutoComplete : TJSHTMLElement;
  Protected
    // Start search
    procedure onSearch(aEvent: TJSEvent);
    function  DoSearchKeyUp(aEvent: TJSKeyBoardEvent): boolean;
    // Local search
    procedure DoLocalSearch(aTerm: String);
    procedure DoMatchSelected(aEvent: TJSEvent);
    function ShowResultMatch(Idx: Integer; aResult: TPDFSearchMatch  ): TJSHTMLElement;
    // Server search
    procedure SetupDatasets;
    procedure DogetURL(Sender: TComponent; aRequest: TDataRequest; var aURL: String);
    // Server word list
    function DoCompleteWord(Event: TEventListenerEvent): boolean;
    procedure DoWordSelected(Event: TJSEvent);
    procedure DoWordsOpen(DataSet: TDataSet);
    // Server index search
    procedure DoIndexSearch(aTerm: String);
    procedure DoOpenResults(DataSet: TDataSet);
    function  ShowServerMatch(aResult: TServerMatch): TJSHTMLElement;
    procedure DoServerMatchSelected(aEvent: TJSEvent);
  Public
    constructor Create(aOwner : TComponent); override;
    procedure BindElements;
    Procedure ShowResultPanel;
    Procedure HideResultPanel;
    Procedure ClearResultPanel;
    Property pdfPanel : TPDFPanel Read FpdfPanel Write FpdfPanel;
    Property LocalCheckboxID : String Read FLocalCheckboxID Write FLocalCheckboxID;
    Property SearchButtonID : String Read FSearchButtonID Write FSearchButtonID;
    Property SearchEditID : String Read FSearchEditID Write FSearchEditID;
    Property DivAutoCompleteID : String Read FDivAutoCompleteID Write FDivAutoCompleteID;
    Property ResultPanelID : String Read FResultPanelID Write FResultPanelID;
    Property SidebarPanelID : String Read FSidebarPanelID Write FSidebarPanelID;
    Property ShowResultsPanelButtonID : String Read fShowResultsPanelButtonID Write FShowResultsPanelButtonID;
  end;

implementation

procedure TServerResultsMap.InitFields;
begin
  fid:=FieldByName('id');
  frank:=FieldByName('rank');
  fpage:=FieldByName('articlePage');
  fissue:=FieldByName('articleIssue');
  fauthor:=FieldByName('articleAuthor');
  ftitle:=FieldByName('articleTitle');
  furl:=FieldByName('url');
  fcontext:=FieldByName('context');
  fdate:=FieldByName('date');
end;

procedure TServerMatch.FromMap(aMap: TServerResultsMap);
begin
  With aMap do
    begin
    ID:=GetFromField(FID,ID);
    Rank:=GetFromField(FRank,Rank);
    Page:=GetFromField(FPage,Page);
    Issue:=GetFromField(FIssue,Issue);
    Author:=GetFromField(FAuthor,Author);
    Title:=GetFromField(FTitle,Title);
    URL:=GetFromField(FURL,URL);
    Context:=GetFromField(FContext,Context);
    Date:=GetFromField(FDate,Date)
    end;
end;

function TRestDataset.DoGetDataProxy: TDataProxy;
begin
  Result:=Connection.DataProxy;
end;


procedure TPDFSearchControl.DogetURL(Sender: TComponent; aRequest: TDataRequest; var aURL: String);

var
  Q : String;

begin
  Q:=encodeURIComponent(FSearchTerm);
  if aRequest.Dataset=FResult then
   aURL:=FConn.BaseURL+'search?m=1&q='+q
 else
   aURL:=FConn.BaseURL+'list?t=contains&m=1&q='+q;
end;

procedure TPDFSearchControl.DoWordSelected(Event: TJSEvent);

begin
  event.PreventDefault;
  edtSearch.value:=event.targetelement.innerText;
  mnuAutoComplete.style.setProperty('display','none');
end;

procedure TPDFSearchControl.DoWordsOpen(DataSet: TDataSet);

Var
  S : String;
  F : TField;
  P : TJSHTMLELement;
  A : TJSHTMLAnchorElement;


begin
  mnuAutoComplete.style.setProperty('display','none');
  mnuAutoComplete.InnerHTML:='<div class="dropdown-content"></div>';
  P:=TJSHTMLELement(mnuAutoComplete.firstElementChild);
  if Dataset.RecordCount<=0 then
    exit;
  F:=Dataset.FieldByName('Word');
  While Not Dataset.EOF do
   begin
   S:=F.AsString;
   a:=TJSHTMLAnchorElement(Document.createElement('a'));
   a.href:='#';
   a.classlist.Add('dropdown-item');
   a.innerText:=s;
   a.dataset['value']:=s;
   a.addEventListener('click',@DoWordSelected);
   P.appendChild(a);
   Dataset.Next;
   end;
  mnuAutoComplete.style.setProperty('display','block');
end;


procedure TPDFSearchControl.DoServerMatchSelected(aEvent: TJSEvent);
var
  aPage : Integer;
  aTitle,aURL : string;
  Src : TPDFSource;

begin
  aEvent.currentTargetElement.classList.add('is-active');
  aPage:=StrToIntDef(String(TJSHTMLElement(aEvent.currentTargetElement).Dataset['page']),-1);
  aTitle:=String(TJSHTMLElement(aEvent.currentTargetElement).Dataset['title']);
  aUrl:=TJSHTMLElement(aEvent.currentTargetElement).Dataset['url'];
  if (aPage<>-1) and (aUrl<>'') then
    begin
    // elf,aTitle,aUrl,aPage
    Src:=TPDFSource.new;
    Src.url:=ServerURL+'pdf/'+aUrl;
    pdfPanel.ShowPDF(Src,aPage);
    HideResultPanel;
    end;
end;


function TPDFSearchControl.DoCompleteWord(Event: TEventListenerEvent): boolean;

  procedure DoServerSearchWord;

  begin
    if Length(edtSearch.Value)>1 then
      begin
      FSearchTerm:=edtSearch.Value;
      FWords.Close;
      FWords.Load([],Nil);
      end;
  end;

begin
  Result:=False;
  if FSearchTimerID<>0 then
    window.clearTimeout(FSearchTimerID);
  FSearchTimerID:=window.SetTimeout(@DoServerSearchWord,200);
end;

function TPDFSearchControl.ShowServerMatch(aResult: TServerMatch
  ): TJSHTMLElement;

Const
  ResultContent =
  '<a class="panel-block result-item"> '+
  '  <div>'+
  '    <span class="tag is-warning">Issue {{issue}}, page {{page}}</span>'+
  '    <p class="title is-5 is-spaced is-marginless">'+
  '    {{title}}'+
  '    </p>'+
  '    <p class="is-marginless">'+
  '    {{author}}'+
  '    </p>'+
  '    <p class="is-marginless">'+
  '    Page: {{page}} {{content}}'+
  '    </p>'+
  '  </div>'+
  '</a>';
  Highlight = '<span class="has-text-weight-bold">{{match}}</span>';

Var
  aReplace, Content, Res : String;

begin
  Result:=TJSHTMLElement(Document.CreateElement('a'));
  Result.Dataset['page']:=IntToStr(aResult.Page);
  Result.Dataset['idx']:=IntToStr(aResult.ID);
  Result.Dataset['title']:=aResult.Title;
  Result.Dataset['url']:=aResult.url;
  Result.ClassName:='panel-block result-item';
  Res:=ResultContent;
  Res:=StringReplace(Res,'{{page}}',IntToStr(aResult.Page),[rfReplaceAll]);
  Res:=StringReplace(Res,'{{author}}',aResult.Author,[rfReplaceAll]);
  Res:=StringReplace(Res,'{{issue}}',aResult.Issue,[rfReplaceAll]);
  Res:=StringReplace(Res,'{{title}}',aResult.Title,[rfReplaceAll]);
  aReplace:=StringReplace(Highlight,'{{match}}',edtSearch.Value,[rfReplaceAll]);
  Content:=StringReplace(aResult.Context,edtSearch.Value,aReplace,[rfReplaceAll]);
  Res:=StringReplace(Res,'{{content}}',Content,[]);
  Writeln('Res : ',Res);
  Result.InnerHTML:=Res;
  Result.AddEventListener('click',@DoServerMatchSelected);
  pnlResults.appendChild(Result);
end;

procedure TPDFSearchControl.DoOpenResults(DataSet: TDataSet);

var
  i : Integer;
  aResult: TServerMatch;
  aMap : TServerResultsMap;
  NoFilter : Boolean;

begin
  I:=0;
  ShowResultPanel;
//  UpdateExpandButton;
//  divWrapper.InnerHTML:='';

  aMap:=TServerResultsMap.Create(Dataset);
  While not aMap.Dataset.EOF do
    begin
    Inc(I);
    aResult.FromMap(aMap);
    ShowServerMatch(aResult);
    Dataset.next;
    end;
end;


procedure TPDFSearchControl.SetupDatasets;

begin
  FConn:=TRESTConnection.Create(Self);
  //  FConn.BaseUrl:='./docsearch.cgi/';
  FConn.BaseURL:=ServerURL;
  FConn.OnGetURL:=@DogetURL;

  FResult:=TRestDataset.Create(Self);
  FResult.Connection:=FConn;
  Fresult.AfterOpen:=@DoOpenResults;
  FWords:=TRestDataset.Create(Self);
  FWords.Connection:=FConn;
  FWords.AfterOpen:=@DoWordsOpen;
end;

constructor TPDFSearchControl.Create(aOwner: TComponent);
begin
  Inherited;
  SetupDatasets;
  DivAutoCompleteID:='mnuAutoComplete';
  SearchEditID:='edtSearch';
  LocalCheckboxID:='cbLocal';
  SearchButtonID:='btnSearch';
  ResultPanelID:='pnlResults';
  SidebarPanelID:='pnlSidebar';
  ShowResultsPanelButtonID:='btnClosePaneImage';
end;

procedure TPDFSearchControl.onSearch(aEvent: TJSEvent);
var
  aterm : string;

begin
  aTerm:=edtSearch.Value;
  if Length(aTerm)<=1 then
    exit;
  if cbLocal.Checked then
    begin
    if not assigned(pdfPanel) then
      exit;
    DoLocalSearch(aTerm);
    end
  else
    DoIndexSearch(aTerm);
end;

procedure TPDFSearchControl.DoMatchSelected(aEvent: TJSEvent);

  procedure MakeInactive(currentValue : TJSNode; currentIndex: NativeInt; list : TJSNodeList);

  begin
    TJSHTMLElement(CurrentValue).classList.add('is-active');
  end;

var
  aPage : Integer;

begin
  pnlResults.querySelectorAll('result-item').foreach(@MakeInactive);
  aEvent.currentTargetElement.classList.add('is-active');
  aPage:=StrToIntDef(String(TJSHTMLElement(aEvent.currentTargetElement).Dataset['page']),-1);
  if aPage<>-1 then
    PDFPanel.queueRenderPage(aPage);
  HideResultPanel;
end;

function TPDFSearchControl.ShowResultMatch(Idx: Integer;
  aResult: TPDFSearchMatch): TJSHTMLElement;

Const
  ResultContent =
  '<span class="panel-icon">'+
  '  <i class="las la-book" aria-hidden="true"></i>'+
  '</span>'+
  '<div class="subtitles">'+
  '  <p class="has-text-weight-semibold">Page %d:</p>'+
  '  <p>'+
  '    <span>%s</span>'+
  '    <span class="has-text-weight-bold">%s</span>'+
  '    <span>%s</span>'+
  '  </p>'+
  '</div>';

begin
  Result:=TJSHTMLElement(Document.CreateElement('a'));
  Result.Dataset['page']:=IntToStr(aResult.Page);
  Result.Dataset['idx']:=IntToStr(Idx);
  Result.ClassName:='panel-block result-item';
  Result.InnerHTML:=Format(ResultContent,[aResult.Page,aResult.Context1,aResult.Text,aResult.Context2]);
  Result.AddEventListener('click',@DoMatchSelected);
  pnlResults.appendChild(Result);
end;


procedure TPDFSearchControl.DoLocalSearch(aTerm : String);

  Function OnHaveResults(aValue : JSValue) : JSValue;

  var
    Results : TPDFSearchResult absolute aValue;
    I : Integer;

  begin
    Result:=Undefined;
    If Length(Results.Matches)>0 then
      begin
      ClearResultPanel;
      For I:=1 to Length(Results.Matches)-1 do
        ShowResultMatch(I,Results.Matches[i]);
      ShowResultPanel;
      end;
  end;

begin
  FSearchTerm:=aTerm;
  if FSearch=Nil then
    FSearch:=TPDFSearch.Create(Self);
  FSearch.PDF:=PDFPanel.PDFDoc;
  FSearch.searchDocument(aTerm)._then(@OnHaveResults);
end;

procedure TPDFSearchControl.DoIndexSearch(aTerm : String);

begin
  FResult.Close;
  FSearchTerm:=aTerm;
  FResult.Load();
end;

function TPDFSearchControl.DoSearchKeyUp(aEvent: TJSKeyBoardEvent): boolean;

begin
  Result:=False;
  if (aEvent.Key<>TJSKeyNames.Enter) then
    exit;
  onSearch(aEvent);
end;

procedure TPDFSearchControl.BindElements;

  function GetHTMLElement(aID : String) : TJSHTMLElement;
  begin
    Result:=TJSHTMLElement(document.getElementById(aID));
    if Not Assigned(Result) then
      Writeln('Element ',aId,' Not found');
  end;

begin
  cbLocal:=TJSHTMLInputElement(GetHTMLElement(LocalCheckboxID));
  btnSearch:=TJSHTMLButtonElement(GetHTMLElement(SearchButtonID));
  btnSearch.addEventListener('click', @onSearch);
  edtSearch:=TJSHTMLInputElement(GetHTMLElement(SearchEditID));
  edtSearch.onkeyup:=@DoSearchKeyUp;
  edtSearch.oninput:=@DoCompleteWord;
  mnuAutoComplete:=GetHTMLElement(DivAutoCompleteID);
  pnlResults:=GetHTMLElement(ResultPanelID);
  pnlSidebar:=GetHTMLElement(SidebarPanelID);
  btnClosePaneImage:=GetHTMLElement(ShowResultsPanelButtonID);
end;

procedure TPDFSearchControl.ShowResultPanel;
begin
  pnlSidebar.ClassList.Remove('sidebar_closed');
  btnClosePaneImage.ClassList.Add('la-arrow-left');
  btnClosePaneImage.ClassList.Remove('la-arrow-right');
end;

procedure TPDFSearchControl.HideResultPanel;
begin
  btnClosePaneImage.ClassList.Remove('la-arrow-left');
  btnClosePaneImage.ClassList.Add('la-arrow-right');
  pnlSidebar.ClassList.add('sidebar_closed');
end;

procedure TPDFSearchControl.ClearResultPanel;

begin
  pnlResults.InnerHTML:='';
end;

end.

